/*
 *
 *  $Id: compat.h 732 2005-11-27 03:30:44Z isely $
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __PVRUSB2_COMPAT_H
#define __PVRUSB2_COMPAT_H

/*

  This header is present to facilitate integration with the v4l core
  source tree.  In video4Linux, the source code in its out-of-tree
  form is not supposed to include linux/version.h directly but rather
  grab compat.h instead which defines lots of stuff to assist with
  backwards compatibility.  When a patch set is created and submitted
  to the main kernel, a script processes out all the places where
  compat.h is included - because in the main kernel such backwards
  compatibility is not needed.

  Because the pvrusb2 driver has similar needs and in an attempt to
  keep the code as similar as possible between v4l and the driver
  itself, we also will include compat.h instead of linux/version.h.
  Thus this header is defined here to link ourself correctly back to
  the needed kernel header(s).

*/

#include <linux/version.h>

/*
   List of driver configuration directives.  These are intended to be
   automatically set up, but can be overridden manaually here if it
   makes sense for your environment.  These are arranged such that the
   state of having "none" of these defined should make sense for
   v4l-cvs.

     PVR2_SUPPRESS_SAA711X - When defined, this will *suppress*
       compilation of code to use the new saa711x module in V4L (which
       is actually saa7115.ko in 2.6.15, **NOT** saa711x.ko in 2.6.15
       which appears to be an inferior incomplete module that I
       believe will disappear soon).  Normally we don't want to
       suppress this, but if the kernel version is earlier than
       2.6.15, then we have to shut this off because otherwise the
       driver will not compile (because the module didn't appear in
       the kernel until 2.6.15).

     PVR2_ENABLE_SAA7115 - When defined, this enables the compilation
       of code to use the saa7115 that comes from ivtv.  Normally we
       always want to enable this (but in v4l-cvs it makes no sense to
       have it around).

     PVR2_EEPROM_INDIRECT - When defined, use indirect method to
       access the device eeprom.  This is technically the cleaner way
       to get at the eeprom data, but unfortunately tveeprom.ko
       doesn't seem to work right when doing this on our behalf.  Thus
       you should probably just never define this.  But if you are
       morbidly curious, you can try it...

     PVR2_EEPROM_IVTV - When defined and only if PVR2_EEPROM_INDIRECT
       is not set (otherwise there is no effect), this causes the
       direct method of tveeprom to be adjusted to be as forgiving as
       possible regarding the version of tveeprom that it is dealing
       with.  This selects the "horrible hack" method referred to in
       the pvrusb2-eeprom.html.  When not defined, we just assume
       we're dealing with whatever tveeprom.ko that V4L has
       implemented and don't make allowances for compatibility
       fracturing brain damage.


   MCI <isely@pobox.com> 26-Nov-2005 - About saa7115 vs saa711x...
   These are two different modules but they serve the same purpose.
   The "saa7115" that we have known all along comes from ivtv, while
   "saa711x" is new and comes from v4l.  Unfortunately their APIs are
   completely different.  The pvrusb2 driver can use either one; the
   results are the same.  If only one choice is enabled here then that
   is all the driver will look for.  If both choices are enabled, then
   the driver will detect which one is loaded and use it.  You *must*
   have one of these modules loaded and operable by this driver in
   order for video processing to function.  Making matters more
   confusing is that within V4L at least for 2.6.15 itself, there are
   two variations of this module, saa711x.ko and saa7115.ko.  The one
   named saa7115.ko *is* actually the saa711x we're talking about
   here.  The one named saa711x.ko is apparently an incomplete dead
   end that AFAIK will be going away soon, whereupon saa7115.ko will
   be renamed on top of it.  Even though it's currently known as
   saa7115.ko, internally it really is saa711x.ko with the entirely
   different API and it identifies itself to outside users as saa711x.
   Short summary: Only load saa7115.ko not saa711x.ko (and that is
   what the driver will attempt for now).  Clear as mud?  Yeah, I
   know.  Very confusing.  This craziness is likely due the ongoing
   merge of ivtv into v4l.  Please don't shoot the messenger :-)


*/


/* Disable use of saa711x if the kernel isn't recent enough */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
#define PVR2_SUPPRESS_SAA711X
#endif

/* Always enable the use of the ivtv saa7115 module */
#define PVR2_ENABLE_SAA7115

/* For stand-alone builds, be as forgiving as possible */
#define PVR2_EEPROM_IVTV

#endif /* __PVRUSB2_COMPAT_H */

/*
  Stuff for Emacs to see, in order to encourage consistent editing style:
  *** Local Variables: ***
  *** mode: c ***
  *** fill-column: 70 ***
  *** tab-width: 8 ***
  *** c-basic-offset: 8 ***
  *** End: ***
  */
